import React, { useState, useEffect, useRef } from "react";
import { Upload, Share, Trash2 } from "lucide-react";
import { apiCall } from "../api/apiClient";
import FilePreviewer from "./preview/FilePreviewer";

interface File {
  id: number;
  user_id: number;
  original_name: string;
  storage_path: string;
  uploaded_at: string;
  size: number;
}

interface FilesComponentProps {
  isDarkMode: boolean;
}

export const FilesComponent: React.FC<FilesComponentProps> = ({ isDarkMode }) => {
  const [files, setFiles] = useState<File[]>([]);
  const [loading, setLoading] = useState(false);
  const [error, setError] = useState<string | null>(null);
  const [success, setSuccess] = useState<string | null>(null);
  const [openDropdown, setOpenDropdown] = useState<number | null>(null);
  const [token, setToken] = useState<string | null>(null);
  const [shareDuration, setShareDuration] = useState(7); // durata di default 7 giorni
  const [previewFile, setPreviewFile] = useState<{ url: string; name: string } | null>(null);
  const [previewBlobUrl, setPreviewBlobUrl] = useState<string | null>(null);
  const [uploadProgress, setUploadProgress] = useState<number | null>(null);
  const [uploadFileName, setUploadFileName] = useState<string | null>(null);
  const [uploadStartTime, setUploadStartTime] = useState<number | null>(null);

  const fileInputRef = useRef<HTMLInputElement>(null);

  // Recupera il token solo una volta al mount o quando cambia
  useEffect(() => {
    const storedToken = localStorage.getItem('litecloud_token'); // <-- usa la chiave corretta
    setToken(storedToken);
  }, []);

  const fetchFiles = async () => {
    if (!token) return; // Non chiamare se il token non è pronto
    try {
      setLoading(true);
      setError(null);
      const response = await apiCall('/api/files');
      const filesData = await response.json();
      setFiles(filesData);
    } catch (err) {
      setError('Failed to fetch files');
    } finally {
      setLoading(false);
    }
  };

  useEffect(() => {
    if (token) fetchFiles();
  }, [token]);

  // Fix gestione click fuori dal dropdown
  useEffect(() => {
    const handleClickOutside = (event: MouseEvent) => {
      // Se il dropdown è aperto e il click NON è dentro un menu dropdown, chiudi
      const dropdowns = document.querySelectorAll('.custom-dropdown');
      let clickedInside = false;
      dropdowns.forEach(dropdown => {
        if (dropdown.contains(event.target as Node)) {
          clickedInside = true;
        }
      });
      if (!clickedInside) {
        setOpenDropdown(null);
      }
    };
    document.addEventListener('mousedown', handleClickOutside);
    return () => document.removeEventListener('mousedown', handleClickOutside);
  }, []);

  const handleFileUpload = async (event: React.ChangeEvent<HTMLInputElement>) => {
    const file = event.target.files?.[0];
    if (!file) return;

    try {
      setLoading(true);
      setError(null);
      setUploadProgress(0);
      setUploadFileName(file.name);
      setUploadStartTime(Date.now());
      const formData = new FormData();
      formData.append('file', file);
      formData.append('filename', file.name);

      if (!token) throw new Error('No auth token found');
      const xhr = new XMLHttpRequest();
      xhr.open('POST', '/api/upload');
      xhr.setRequestHeader('Authorization', `Bearer ${token}`);
      xhr.upload.onprogress = (e) => {
        if (e.lengthComputable) {
          setUploadProgress(Math.round((e.loaded / e.total) * 100));
        }
      };
      xhr.onload = async () => {
        if (xhr.status >= 200 && xhr.status < 300) {
          setSuccess('File uploaded successfully!');
          fetchFiles();
        } else {
          setError('Upload failed');
        }
        setLoading(false);
        setUploadProgress(null);
        setUploadFileName(null);
        setUploadStartTime(null);
        if (fileInputRef.current) fileInputRef.current.value = '';
      };
      xhr.onerror = () => {
        setError('Failed to upload file');
        setLoading(false);
        setUploadProgress(null);
        setUploadFileName(null);
        setUploadStartTime(null);
        if (fileInputRef.current) fileInputRef.current.value = '';
      };
      xhr.send(formData);
    } catch (err) {
      setError('Failed to upload file - ' + (err instanceof Error ? err.message : 'Unknown error'));
      setLoading(false);
      setUploadProgress(null);
      setUploadFileName(null);
      setUploadStartTime(null);
      if (fileInputRef.current) fileInputRef.current.value = '';
    }
  };

  const handleFileAction = async (
    action: 'share' | 'delete',
    fileId: number,
    duration?: number
  ) => {
    try {
      setLoading(true);
      setError(null);
      if (action === 'share') {
        const response = await apiCall('/api/share', {
          method: 'POST',
          body: JSON.stringify({
            file_id: fileId,
            expires_in_days: duration || 7
          }),
        });
        const shareId = await response.json();
        setSuccess(`Share link created: /shared/${shareId}`);
      } else if (action === 'delete') {
        await apiCall(`/api/files/${fileId}`, {
          method: 'DELETE',
        });
        setSuccess('File deleted successfully');
        fetchFiles();
      }
    } catch (err) {
      setError(`Failed to ${action} file`);
    } finally {
      setLoading(false);
      setOpenDropdown(null);
    }
  };

  const handleDownload = async (file: File) => {
    try {
      setLoading(true);
      setError(null);
      if (!token) throw new Error('No auth token found');
      const response = await fetch(`/api/files/${file.id}`, {
        headers: {
          'Authorization': `Bearer ${token}`,
        },
      });
      if (!response.ok) throw new Error('Download failed');
      const blob = await response.blob();
      const url = window.URL.createObjectURL(blob);
      const a = document.createElement('a');
      a.href = url;
      a.download = file.original_name;
      document.body.appendChild(a);
      a.click();
      a.remove();
      window.URL.revokeObjectURL(url);
    } catch (err) {
      setError('Failed to download file');
    } finally {
      setLoading(false);
      setOpenDropdown(null);
    }
  };

  // Funzione per ottenere e decriptare il file dal server
  const handlePreviewFile = async (file: File) => {
    try {
      setLoading(true);
      setError(null);
      if (!token) throw new Error('No auth token found');
      const response = await fetch(`/api/files/${file.id}`, {
        headers: {
          'Authorization': `Bearer ${token}`,
        },
      });
      if (!response.ok) throw new Error('Impossibile scaricare il file');
      const blob = await response.blob();
      const blobUrl = URL.createObjectURL(blob);
      setPreviewBlobUrl(blobUrl);
      setPreviewFile({ url: blobUrl, name: file.original_name });
    } catch (err) {
      setError('Errore durante il download per anteprima');
    } finally {
      setLoading(false);
    }
  };

  // Pulisci la URL temporanea quando chiudi la preview
  const handleClosePreview = () => {
    if (previewBlobUrl) {
      URL.revokeObjectURL(previewBlobUrl);
      setPreviewBlobUrl(null);
    }
    setPreviewFile(null);
  };

  const formatDate = (dateString: string) => {
    const now = new Date();
    const date = new Date(dateString);
    const diff = (now.getTime() - date.getTime()) / 1000;

    const isSameDay = (d1: Date, d2: Date) =>
      d1.getFullYear() === d2.getFullYear() &&
      d1.getMonth() === d2.getMonth() &&
      d1.getDate() === d2.getDate();

    const yesterday = new Date();
    yesterday.setDate(now.getDate() - 1);

    if (diff < 10) return "now";

    if (isSameDay(date, yesterday)) {
      return `yesterday at ${date.toLocaleTimeString()}`;
    }

    return date.toLocaleString();
  };


  const formatFileSize = (bytes: number) => {
    const sizes = ['B', 'KB', 'MB', 'GB'];
    if (bytes === 0) return '0 B';
    const i = Math.floor(Math.log(bytes) / Math.log(1024));
    return Math.round(bytes / Math.pow(1024, i) * 100) / 100 + ' ' + sizes[i];
  };

  // Funzione per ottenere la URL del file (puoi adattare se serve fetch protetto)

  // Calcola il tempo rimanente stimato
  const getEstimatedTimeLeft = () => {
    if (uploadProgress === null || uploadProgress === 0 || !uploadStartTime) return null;
    const now = Date.now();
    const elapsed = (now - uploadStartTime) / 1000; // in secondi
    const estimatedTotal = elapsed / (uploadProgress / 100);
    const remaining = estimatedTotal - elapsed;
    if (!isFinite(remaining) || remaining < 0) return null;
    if (remaining < 60) return `${Math.round(remaining)}s`;
    return `${Math.floor(remaining / 60)}m ${Math.round(remaining % 60)}s`;
  };

  return (
    <div className="component-container">
      {/* Success/Error Messages */}
      {error && (
        <div className="notification notification-error">
          {error}
          <button onClick={() => setError(null)} className="notification-close">×</button>
        </div>
      )}
      {success && (
        <div className="notification notification-success">
          {success}
          <button onClick={() => setSuccess(null)} className="notification-close">×</button>
        </div>
      )}

      {/* Modal anteprima file */}
      {previewFile && (
        <div className="modal-overlay" style={{ position: 'fixed', top:0, left:0, right:0, bottom:0, background: 'rgba(0,0,0,0.85)', zIndex: 1000, display: 'flex', alignItems: 'stretch', justifyContent: 'center' }}>
          <div className="modal-content" style={{ background: isDarkMode ? '#222' : '#fff', padding: 0, borderRadius: 0, width: '100vw', height: '100vh', maxWidth: '100vw', maxHeight: '100vh', overflow: 'auto', position: 'relative', display: 'flex', flexDirection: 'column' }}>
            <button onClick={handleClosePreview} style={{ position: 'absolute', top: 16, right: 24, fontSize: 36, background: 'none', border: 'none', color: isDarkMode ? '#fff' : '#222', cursor: 'pointer', zIndex: 10 }}>×</button>
            <div style={{ flex: 1, display: 'flex', alignItems: 'center', justifyContent: 'center', minHeight: 0, minWidth: 0 }}>
              <FilePreviewer fileUrl={previewFile.url} fileName={previewFile.name} fullscreen />
            </div>
          </div>
        </div>
      )}

      <div className="component-header">
        <h1 className={`component-title ${isDarkMode ? 'text-white' : 'text-gray-800'}`}>
          My Files
        </h1>
        <input
          ref={fileInputRef}
          type="file"
          onChange={handleFileUpload}
          className="hidden"
        />
        <button
          onClick={() => fileInputRef.current?.click()}
          disabled={loading}
          className={`upload-button ${
            isDarkMode 
              ? 'upload-button-dark' 
              : 'upload-button-light'
          }`}
        >
          <Upload size={18} />
        </button>
      </div>

      {loading && (
        <div className="loading-center">
          <div className="loading-spinner-small"></div>
        </div>
      )}

      <div className={`table-container ${isDarkMode ? 'table-container-dark' : 'table-container-light'}`}>
        <table className="data-table full-width-table">
          <thead className={isDarkMode ? 'table-header-dark' : 'table-header-light'}>
            <tr>
              <th className="table-header-cell">Name</th>
              <th className="table-header-cell">Size</th>
              <th className="table-header-cell">Uploaded</th>
              <th className="table-header-cell actions-header-cell"></th> {/* Nessuna scritta */}
            </tr>
          </thead>
          <tbody className={`table-body ${isDarkMode ? 'table-body-dark' : 'table-body-light'}`}> 
            {files.map((file) => (
              <tr key={file.id} className={isDarkMode ? 'table-row-dark' : 'table-row-light'}>
                <td className={`table-cell ${isDarkMode ? 'text-white' : 'text-gray-900'}`}
                  style={{ cursor: 'pointer', textDecoration: 'underline' }}
                  onClick={() => handlePreviewFile(file)}
                  title="Anteprima file"
                >
                  {file.original_name}
                </td>
                <td className={`table-cell ${isDarkMode ? 'text-gray-300' : 'text-gray-500'}`}>{formatFileSize(file.size)}</td>
                <td className={`table-cell ${isDarkMode ? 'text-gray-300' : 'text-gray-500'}`}>{formatDate(file.uploaded_at)}</td>
                <td className="table-cell relative actions-cell">
                  <button
                    onClick={() => setOpenDropdown(openDropdown === file.id ? null : file.id)}
                    className={`actions-button ellipsis-button ${isDarkMode ? 'actions-button-dark' : 'actions-button-light'}`}
                    aria-label="Actions"
                  >
                    <span className="ellipsis-dot"></span>
                    <span className="ellipsis-dot"></span>
                    <span className="ellipsis-dot"></span>
                  </button>
                  {openDropdown === file.id && (
                    <div className={`dropdown-menu ${isDarkMode ? 'dropdown-menu-dark' : 'dropdown-menu-light'} custom-dropdown`}>
                      <div className="dropdown-section">
                        <label className="dropdown-label">Share duration:</label>
                        <select
                          className="dropdown-select"
                          value={shareDuration}
                          onChange={e => setShareDuration(Number(e.target.value))}
                        >
                          <option value={1}>1 day</option>
                          <option value={3}>3 days</option>
                          <option value={7}>7 days</option>
                        </select>
                      </div>
                      <button
                        onClick={() => handleFileAction('share', file.id, shareDuration)}
                        className={`dropdown-item ${
                          isDarkMode ? 'dropdown-item-dark' : 'dropdown-item-light'
                        }`}
                      >
                        <Share size={16} />
                        Share
                      </button>
                      <button
                        onClick={() => handleFileAction('delete', file.id)}
                        className="dropdown-item dropdown-item-danger"
                      >
                        <Trash2 size={16} />
                        Delete
                      </button>
                      <button
                        onClick={() => handleDownload(file)}
                        className={`dropdown-item ${isDarkMode ? 'dropdown-item-dark' : 'dropdown-item-light'}`}
                      >
                        Download
                      </button>
                    </div>
                  )}
                </td>
              </tr>
            ))}
          </tbody>
        </table>
        
        {files.length === 0 && !loading && (
          <div className={`empty-state ${isDarkMode ? 'text-gray-400' : 'text-gray-500'}`}>
            No files uploaded yet
          </div>
        )}
      </div>

      {/* Modal/progressbar upload */}
      {uploadProgress !== null && uploadFileName && (
        <div className="upload-progress-popout">
          <div className="upload-progress-content">
            <div className="upload-progress-filename">{uploadFileName}</div>
            <div className="upload-progress-bar-container">
              <svg className="upload-progress-svg" viewBox="0 0 40 40">
                <circle className="upload-progress-bg" cx="20" cy="20" r="18" />
                <circle
                  className="upload-progress-bar"
                  cx="20"
                  cy="20"
                  r="18"
                  strokeDasharray={2 * Math.PI * 18}
                  strokeDashoffset={2 * Math.PI * 18 * (1 - uploadProgress / 100)}
                />
                <text x="50%" y="54%" textAnchor="middle" className="upload-progress-text">{uploadProgress}%</text>
              </svg>
            </div>
            <div className="upload-progress-eta">
              {getEstimatedTimeLeft() ? `~${getEstimatedTimeLeft()} left` : ''}
            </div>
          </div>
        </div>
      )}
    </div>
  );
};